@@ -4,20 +4,21 @@
 
 from collections import namedtuple
 from numbers import Integral
-from typing import Optional, Union, List, Tuple, Callable
+from typing import Optional, Union, Callable
 
 import numpy as np
 
-from ._koopman import CovarianceKoopmanModel
-from ..base import Estimator, Transformer
+from ._koopman import CovarianceKoopmanModel, KoopmanObservable
+from ..base import EstimatorTransformer
 from ..basis import Identity
 from ..covariance import Covariance, CovarianceModel
 from ..numeric import spd_inv_split
-
-__all__ = ['VAMP']
+from ..util.decorators import deprecated_method
+from ..util.types import to_dataset
+from ..util.validation import DeprecatedCKValidator
 
 
-class VAMP(Estimator, Transformer):
+class VAMP(EstimatorTransformer):
     r"""Variational approach for Markov processes (VAMP).
 
     The implementation is based on :footcite:`wu2020variational`, :footcite:`noe2015kinetic`.
@@ -166,12 +167,6 @@ class VAMP(Estimator, Transformer):
     _DiagonalizationResults = namedtuple("DiagonalizationResults", ['rank0', 'rankt', 'singular_values',
                                                                     'left_singular_vecs', 'right_singular_vecs'])
 
-    @staticmethod
-    def _cumvar(singular_values) -> np.ndarray:
-        cumvar = np.cumsum(singular_values ** 2)
-        cumvar /= cumvar[-1]
-        return cumvar
-
     @staticmethod
     def _decomposition(covariances, epsilon, scaling, dim, var_cutoff) -> _DiagonalizationResults:
         """Performs SVD on covariance matrices and save left, right singular vectors and values in the model."""
@@ -227,7 +222,7 @@ class VAMP(Estimator, Transformer):
             covariances = covariances.fetch_model()
         return covariances
 
-    def partial_fit(self, data: Tuple[np.ndarray, np.ndarray]):
+    def partial_fit(self, data):
         r""" Updates the covariance estimates through a new batch of data.
 
         Parameters
@@ -241,11 +236,11 @@ class VAMP(Estimator, Transformer):
         self : VAMP
             Reference to self.
         """
-        if self.lagtime is None:
-            raise ValueError("Calling partial fit requires that a lagtime is set.")
         if self._covariance_estimator is None:
             self._covariance_estimator = self.covariance_estimator(lagtime=self.lagtime)
-        self._covariance_estimator.partial_fit((self.observable_transform(data[0]), self.observable_transform(data[1])))
+        x, y = to_dataset(data, lagtime=self.lagtime)[:]
+        self._covariance_estimator.partial_fit((self.observable_transform(x),
+                                                self.observable_transform(y)))
         return self
 
     def fit_from_covariances(self, covariances: Union[Covariance, CovarianceModel]):
@@ -267,15 +262,14 @@ class VAMP(Estimator, Transformer):
         self._model = self._decompose(covariances)
         return self
 
-    def fit_from_timeseries(self, data: Union[np.ndarray, List[np.ndarray]],
-                            weights=None):
+    def fit_from_timeseries(self, data, weights=None):
         r""" Estimates a :class:`CovarianceKoopmanModel` directly from time-series data using the :class:`Covariance`
         estimator. For parameters `dim`, `scaling`, `epsilon`.
 
         Parameters
         ----------
-        data : (T, n) ndarray or list thereof
-            Input time-series.
+        data
+            Input data, see :meth:`to_dataset <deeptime.util.types.to_dataset>` for options.
         weights
             See the :class:`Covariance <deeptime.covariance.Covariance>` estimator.
 
@@ -284,8 +278,11 @@ class VAMP(Estimator, Transformer):
         self : VAMP
             Reference to self.
         """
+        dataset = to_dataset(data, lagtime=self.lagtime)
         self._covariance_estimator = self.covariance_estimator(lagtime=self.lagtime)
-        covariances = self._covariance_estimator.fit(self.observable_transform(data), weights=weights).fetch_model()
+        x, y = dataset[:]
+        transformed = (self.observable_transform(x), self.observable_transform(y))
+        covariances = self._covariance_estimator.partial_fit(transformed, weights=weights).fetch_model()
         return self.fit_from_covariances(covariances)
 
     @property
@@ -407,6 +404,20 @@ class VAMP(Estimator, Transformer):
         -------
         self : VAMP
             Reference to self.
+
+        Notes
+        -----
+        If you are running into memory problems for potentially multiple trajectories you can decrease memory load
+        by using :meth:`partial_fit` on individual trajectories or in conjunction with
+        :meth:`timeshifted_split <deeptime.util.data.timeshifted_split>`:
+
+        >>> import numpy as np
+        >>> from deeptime.decomposition import TICA
+        >>> from deeptime.util.data import timeshifted_split
+        >>> estimator = TICA(dim=1)
+        >>> for X, Y in timeshifted_split(np.ones(shape=(100, 5)), lagtime=1, chunksize=40):  # doctest: +SKIP
+        ...     estimator.partial_fit((X, Y))
+        >>> joint_model = estimator.fetch_model()
         """
         if isinstance(data, (Covariance, CovarianceModel)):
             self.fit_from_covariances(data)
@@ -449,3 +460,31 @@ class VAMP(Estimator, Transformer):
             self._model = self._decompose(self._covariance_estimator.fetch_model())
             self._covariance_estimator = None
         return self._model
+
+    @deprecated_method("Deprecated in v0.4.1 and will be removed soon, please use model.ck_test.")
+    def chapman_kolmogorov_validator(self, mlags, test_model: CovarianceKoopmanModel = None,
+                                     n_observables=None, observables='phi', statistics='psi'):
+        r""" Replaced by `deeptime.decomposition.CovarianceKoopmanModel.ck_test`. """
+        test_model = self.fetch_model() if test_model is None else test_model
+        assert test_model is not None, "We need a test model via argument or an estimator which was already " \
+                                       "fit to data."
+
+        def fit_for_lag(data, lagtime):
+            est = VAMP(lagtime=lagtime, dim=self.dim, var_cutoff=self.var_cutoff, scaling=self.scaling,
+                       epsilon=self.epsilon, observable_transform=self.observable_transform)
+            return est.fit(data).fetch_model()
+
+        if isinstance(observables, str) and observables == 'phi':
+            observables = test_model.singular_vectors_right[:, :n_observables]
+            observables_mean_free = True
+        else:
+            observables_mean_free = False
+
+        if isinstance(statistics, str) and statistics == 'psi':
+            statistics = test_model.singular_vectors_left[:, :n_observables]
+            statistics_mean_free = True
+        else:
+            statistics_mean_free = False
+
+        observable = KoopmanObservable(observables, statistics, observables_mean_free, statistics_mean_free)
+        return DeprecatedCKValidator(self, fit_for_lag, mlags, observable, test_model)
