@@ -12,8 +12,7 @@ Maximum-likelihood MSMs (ML-MSM) and Bayesian sampling
     MarkovStateModelCollection
 
     BayesianMSM
-    BayesianPosterior
-    QuantityStatistics
+    BayesianMSMPosterior
 
 Observable operator model MSMs (OOMs)
 -------------------------------------
@@ -36,12 +35,12 @@ Augmented markov models (AMMs)
 """
 import logging
 
-from .markov_state_model import MarkovStateModel, MarkovStateModelCollection
-from .maximum_likelihood_msm import MaximumLikelihoodMSM
-from .bayesian_msm import BayesianMSM, BayesianPosterior
-from .koopman_reweighted_msm import KoopmanReweightedMSM, OOMReweightedMSM
-from .augmented_msm import AugmentedMSMEstimator, AugmentedMSM, AMMOptimizerState
-from ...util.stats import QuantityStatistics
+from ._markov_state_model import MarkovStateModel, MarkovStateModelCollection
+from ._maximum_likelihood_msm import MaximumLikelihoodMSM
+from ._bayesian_msm import BayesianMSM, BayesianMSMPosterior
+from ._koopman_reweighted_msm import KoopmanReweightedMSM, OOMReweightedMSM
+from ._augmented_msm import AugmentedMSMEstimator, AugmentedMSM, AMMOptimizerState
+from .tram import TRAM, TRAMDataset, TRAMModel
 
 # set up null handler
 logging.getLogger(__name__).addHandler(logging.NullHandler())
