@@ -1,13 +1,14 @@
 from math import sqrt
-from typing import Optional, Callable
+from typing import Optional, Callable, Union, List
 
 import numpy as np
 
+from .._transition_counting import TransitionCountEstimator
 from ...base import Estimator
 from ...numeric import is_square_matrix
-from .. import TransitionCountEstimator
-from .._base import _MSMBaseEstimator, BayesianPosterior, MembershipsChapmanKolmogorovValidator
+from .._base import _MSMBaseEstimator, BayesianMSMPosterior
 from . import MarkovStateModel, MaximumLikelihoodMSM
+from ...util.decorators import deprecated_method
 
 __author__ = 'noe, marscher, clonker'
 
@@ -37,17 +38,17 @@ class BayesianMSM(_MSMBaseEstimator):
         this case python sparse matrices will be returned by the corresponding functions instead of numpy arrays.
         This behavior is suggested for very large numbers of states (e.g. > 4000) because it is likely to be much
         more efficient.
-    confidence : float, optional, default=0.954
-        Confidence interval. By default two sigma (95.4%) is used. Use 68.3% for one sigma, 99.7% for three sigma.
     maxiter : int, optional, default=1000000
         Optional parameter with reversible = True, sets the maximum number of iterations before the transition
         matrix estimation method exits.
-    maxerr : float, optional, default = 1e-8
+    maxerr : float, optional, default=1e-8
         Optional parameter with reversible = True. Convergence tolerance for transition matrix estimation. This
         specifies the maximum change of the Euclidean norm of relative stationary probabilities
         (:math:`x_i = \sum_k x_{ik}`). The relative stationary probability changes
         :math:`e_i = (x_i^{(1)} - x_i^{(2)})/(x_i^{(1)} + x_i^{(2)})` are used in order to track changes in small
         probabilities. The Euclidean norm of the change vector, :math:`|e_i|_2`, is compared to maxerr.
+    lagtime : int, optional, default=None
+        The lagtime that is used when fitting directly from discrete trajectories.
 
     References
     ----------
@@ -76,9 +77,9 @@ class BayesianMSM(_MSMBaseEstimator):
      [ 0.38169055  0.          0.61830945]
      [ 0.12023989  0.23690297  0.64285714]]
 
-    Furthermore the bayesian MSM posterior returned by :meth:`fetch_model` is able to
+    Furthermore, the bayesian MSM posterior returned by :meth:`fetch_model` is able to
     compute the probability distribution and statistical models of all methods
-    that are offered by the MSM object. This works as follows. The :meth:`BayesianPosterior.gather_stats` method
+    that are offered by the MSM object. This works as follows. The :meth:`BayesianMSMPosterior.gather_stats` method
     takes as argument the method you want to evaluate and then returns a statistics summary over requested quantity:
 
     >>> print(mm.gather_stats('transition_matrix').mean)  # doctest: +SKIP
@@ -130,14 +131,14 @@ class BayesianMSM(_MSMBaseEstimator):
 
     def __init__(self, n_samples: int = 100, n_steps: int = None, reversible: bool = True,
                  stationary_distribution_constraint: Optional[np.ndarray] = None,
-                 sparse: bool = False, confidence: float = 0.954, maxiter: int = int(1e6), maxerr: float = 1e-8):
+                 sparse: bool = False, maxiter: int = int(1e6), maxerr: float = 1e-8, lagtime: Optional[int] = None):
         super(BayesianMSM, self).__init__(reversible=reversible, sparse=sparse)
         self.stationary_distribution_constraint = stationary_distribution_constraint
         self.maxiter = maxiter
         self.maxerr = maxerr
         self.n_samples = n_samples
         self.n_steps = n_steps
-        self.confidence = confidence
+        self.lagtime = lagtime
 
     @property
     def stationary_distribution_constraint(self) -> Optional[np.ndarray]:
@@ -162,18 +163,18 @@ class BayesianMSM(_MSMBaseEstimator):
             value = np.copy(value) / np.sum(value)
         self._stationary_distribution_constraint = value
 
-    def fetch_model(self) -> Optional[BayesianPosterior]:
+    def fetch_model(self) -> Optional[BayesianMSMPosterior]:
         r"""
         Yields the model that was estimated the most recent.
 
         Returns
         -------
-        model : BayesianPosterior or None
+        model : BayesianMSMPosterior or None
             The estimated model or None if fit was not called.
         """
         return self._model
 
-    def fit(self, data, callback: Callable = None):
+    def fit(self, data, callback: Callable = None, **kw):
         """
         Performs the estimation on either a count matrix or a previously estimated TransitionCountModel.
 
@@ -185,36 +186,31 @@ class BayesianMSM(_MSMBaseEstimator):
         callback: callable, optional, default=None
             Function to be called to indicate progress of sampling.
 
+        Other Parameters
+        ----------------
+        ignore_counting_mode : bool, optional, default=False
+            Method does not raise if counting mode isn't of the "effective" family. Use with caution.
+
         Returns
         -------
         self : BayesianMSM
             Reference to self.
         """
-        from deeptime.markov import TransitionCountModel
-        if isinstance(data, TransitionCountModel) and data.counting_mode is not None \
-                and "effective" not in data.counting_mode:
-            raise ValueError("The transition count model was not estimated using an effective counting method, "
-                             "therefore counts are likely to be strongly correlated yielding wrong confidences.")
-
         if isinstance(data, Estimator):
             if data.has_model:
                 data = data.fetch_model()
             else:
                 raise ValueError("Can only use estimators as input if they have been fit previously.")
 
+        from deeptime.markov import TransitionCountModel
         if isinstance(data, TransitionCountModel) or is_square_matrix(data):
-            msm = MaximumLikelihoodMSM(
-                reversible=self.reversible, stationary_distribution_constraint=self.stationary_distribution_constraint,
-                sparse=self.sparse, maxiter=self.maxiter, maxerr=self.maxerr
-            ).fit(data).fetch_model()
+            return self.fit_from_counts(data)
         elif isinstance(data, MarkovStateModel):
-            msm = data
+            return self.fit_from_msm(data, callback=callback, **kw)
         else:
-            raise ValueError("Unsupported input data, can only be count matrix (or TransitionCountModel, "
-                             "TransitionCountEstimator) or a MarkovStateModel instance or an estimator producing "
-                             "Markov state models.")
-
-        return self.fit_from_msm(msm, callback=callback)
+            if not self.lagtime and 'lagtime' not in kw.keys():
+                raise ValueError("To fit directly from a discrete timeseries, a lagtime must be provided!")
+            return self.fit_from_discrete_timeseries(data, kw.pop('lagtime', self.lagtime), callback=callback, **kw)
 
     def sample(self, prior: MarkovStateModel, n_samples: int, n_steps: Optional[int] = None, callback=None):
         r""" Performs sampling based on a prior.
@@ -244,9 +240,8 @@ class BayesianMSM(_MSMBaseEstimator):
         >>> import deeptime as dt
         >>> dtrajs = [np.array([0,1,2,2,2,2,1,2,2,2,1,0,0,0,0,0,0,0]),
         ...           np.array([0,0,0,0,1,1,2,2,2,2,2,2,2,1,0,0])]
-        >>> prior = dt.markov.msm.MaximumLikelihoodMSM().fit(dtrajs, lagtime=1)
-        >>> estimator = dt.markov.msm.BayesianMSM()
-        >>> posterior = estimator.fit(prior).fetch_model()
+        >>> estimator = dt.markov.msm.BayesianMSM(lagtime=1)
+        >>> posterior = estimator.fit_fetch(dtrajs)
         >>> n_samples = len(posterior.samples)
         >>> posterior.samples.extend(estimator.sample(posterior.prior, n_samples=23))
         >>> assert len(posterior.samples) == n_samples + 23
@@ -275,7 +270,7 @@ class BayesianMSM(_MSMBaseEstimator):
         ]
         return samples
 
-    def fit_from_msm(self, msm: MarkovStateModel, callback=None):
+    def fit_from_msm(self, msm: MarkovStateModel, callback=None, **kw):
         r""" Fits a bayesian posterior from a given Markov state model. The MSM must contain a count model to be able
         to produce confidences. Note that the count model should be produced using effective counting, otherwise
         counts are correlated and computed confidences are wrong.
@@ -287,6 +282,11 @@ class BayesianMSM(_MSMBaseEstimator):
         callback : callable, optional, default=None
             Function to be called to indicate progress of sampling.
 
+        Other Parameters
+        ----------------
+        ignore_counting_mode : bool, optional, default=False
+            Method does not raise if counting mode isn't of the "effective" family. Use with caution.
+
         Returns
         -------
         self : BayesianMSM
@@ -295,45 +295,83 @@ class BayesianMSM(_MSMBaseEstimator):
         if not msm.has_count_model:
             raise ValueError("Can only sample confidences with a count model. The counting mode should be 'effective'"
                              " to avoid correlations between counts and therefore wrong confidences.")
+        if not kw.get("ignore_counting_mode", False) \
+                and msm.count_model.counting_mode is not None and "effective" not in msm.count_model.counting_mode:
+            raise ValueError("The transition count model was not estimated using an effective counting method, "
+                             "therefore counts are likely to be strongly correlated yielding wrong confidences. "
+                             "To ignore this, set `ignore_counting_mode` to True in the call to `fit`.")
         # use the same count matrix as the MLE. This is why we have effective as a default
         samples = self.sample(msm, self.n_samples, self.n_steps, callback)
-        self._model = BayesianPosterior(prior=msm, samples=samples)
+        self._model = BayesianMSMPosterior(prior=msm, samples=samples)
         return self
 
-    def chapman_kolmogorov_validator(self, n_metastable_sets: int, mlags, test_model: BayesianPosterior = None):
-        r"""Returns a Chapman-Kolmogorov validator based on this estimator and a test model.
+    def fit_from_discrete_timeseries(self, discrete_timeseries: Union[np.ndarray, List[np.ndarray]],
+                                     lagtime: int = None, count_mode: str = 'effective', callback=None, **kw):
+        r""" Fits a BayesianMSM directly on timeseries data.
 
         Parameters
         ----------
-        n_metastable_sets : int
-            Number of metastable sets to project the state space down to.
-        mlags : int or range or list
-            Multiple of lagtimes of the test_model to test against.
-        test_model : BayesianPosterior, optional, default=None
-            The model that is tested. If not provided, uses this estimator's encapsulated model.
+        discrete_timeseries : list of ndarray
+            Discrete trajectories.
+        lagtime : int, optional, default=None
+            The lagtime that is used for estimation. If None, uses the instance's lagtime attribute.
+        count_mode : str, default='effective'
+            The counting mode. Should be of the `effective` kind, otherwise the results may be heavily biased.
+        callback : callable, optional, default=None
+            Function to be called to indicate progress of sampling.
+        **kw
+            Optional keyword parameters.
 
         Returns
         -------
-        validator : markov.MembershipsChapmanKolmogorovValidator
-            The validator.
+        self : BayesianMSM
+            Reference to self.
         """
+        counts = TransitionCountEstimator(lagtime=lagtime, count_mode=count_mode,
+                                          sparse=self.sparse).fit_fetch(discrete_timeseries).submodel_largest()
+        return self.fit_from_counts(counts, callback=callback, **kw)
+
+    def fit_from_counts(self, counts, callback=None, **kw):
+        r"""Fits a bayesian MSM on a count model or a count matrix.
+
+        Parameters
+        ----------
+        counts : TransitionCountModel or (n, n) ndarray
+            The transition counts.
+        callback : callable, optional, default=None
+            Function that is called to indicate progress of sampling.
+        **kw
+            Optional keyword parameters.
+
+        Returns
+        -------
+        self : BayesianMSM
+            Reference to self.
+        """
+        msm = MaximumLikelihoodMSM(
+            reversible=self.reversible, stationary_distribution_constraint=self.stationary_distribution_constraint,
+            sparse=self.sparse, maxiter=self.maxiter, maxerr=self.maxerr
+        ).fit(counts).fetch_model()
+        return self.fit_from_msm(msm, callback=callback, **kw)
+
+    @deprecated_method("Deprecated in v0.4.1 and will be removed soon, please use model.ck_test.")
+    def chapman_kolmogorov_validator(self, n_metastable_sets: int, mlags, test_model=None):
+        r""" Replaced by `deeptime.markov.msm.BayesianMSMPosterior.ck_test`. """
         test_model = self.fetch_model() if test_model is None else test_model
         assert test_model is not None, "We need a test model via argument or an estimator which was already" \
                                        "fit to data."
         prior = test_model.prior
         assert prior.has_count_model, "The test model needs to have a count model, i.e., be estimated from data."
         pcca = prior.pcca(n_metastable_sets)
-        reference_lagtime = prior.count_model.lagtime
-        return BayesianMSMChapmanKolmogorovValidator(test_model, self, pcca.memberships, reference_lagtime, mlags)
+        from deeptime.markov._observables import MembershipsObservable
+        obs = MembershipsObservable(test_model, pcca)
+        from deeptime.util.validation import DeprecatedCKValidator
 
+        def fit_for_lag(data, lag):
+            from deeptime.markov import TransitionCountEstimator
+            counting_mode = test_model.prior.count_model.counting_mode
+            counts = TransitionCountEstimator(lag, counting_mode).fit(data, n_jobs=1).fetch_model().submodel_largest()
+            return self.fit(counts).fetch_model()
 
-def _ck_estimate_model_for_lag(estimator, model, data, lagtime):
-    counting_mode = model.prior.count_model.counting_mode
-    counts = TransitionCountEstimator(lagtime, counting_mode).fit(data, n_jobs=1).fetch_model().submodel_largest()
-    return estimator.fit(counts).fetch_model()
+        return DeprecatedCKValidator(self, fit_for_lag, mlags, obs, test_model)
 
-
-class BayesianMSMChapmanKolmogorovValidator(MembershipsChapmanKolmogorovValidator):
-
-    def fit(self, data, n_jobs=None, progress=None, **kw):
-        return super().fit(data, n_jobs, progress, estimate_model_for_lag=_ck_estimate_model_for_lag, **kw)
\ No newline at end of file
