@@ -2,6 +2,7 @@ import numpy as np
 import scipy as sp
 import warnings
 
+from deeptime.base import Dataset
 from deeptime.util import types, callbacks
 from deeptime.util.decorators import cached_property
 from deeptime.markov import TransitionCountEstimator, TransitionCountModel
@@ -86,7 +87,7 @@ def _invalidate_caches():
             member.invalidate()
 
 
-class TRAMDataset:
+class TRAMDataset(Dataset):
     r""" Dataset for organizing data and obtaining properties from data that are needed for TRAM.
     The minimum required parameters for constructing a TRAMDataset are the `dtrajs` and `bias_matrices`. In this case,
     `ttrajs` are inferred from the shape of the `dtrajs`, by assuming each trajectory in `dtrajs` corresponds to a
@@ -143,7 +144,6 @@ class TRAMDataset:
     :class:`TRAM <deeptime.markov.msm.TRAM>`, :class:`TRAMModel <deeptime.markov.msm.TRAMModel>`
 
     """
-
     def __init__(self, dtrajs, bias_matrices, ttrajs=None, n_therm_states=None, n_markov_states=None, lagtime=1,
                  count_mode='sliding'):
         self.lagtime = lagtime
@@ -173,13 +173,35 @@ class TRAMDataset:
     #: All possible connectivity modes
     connectivity_options = ["post_hoc_RE", "BAR_variance", "summed_count_matrix", None]
 
+    def __len__(self):
+        return np.sum([len(traj) for traj in self.dtrajs])
+
+    def __getitem__(self, indices):
+        traj, n = indices
+        return self.dtrajs[traj][n], self.bias_matrices[traj][n]
+
+    def setflags(self, write=True):
+        [traj.setflags(write=write) for traj in self.dtrajs]
+        [bias_matrix.setflags(write=write) for bias_matrix in self.bias_matrices]
+
     @property
     def tram_input(self):
         r""" The TRAMInput object containing the data needed for estimation.
-        For estimation purposes, it does not matter which thermodynamic state each sample was sampled at. The dtrajs and
-        bias_matrices are therefore flattened along the first dimension, to speed up estimation. """
-        return tram.TRAMInput(self.state_counts, self.transition_counts,
-                              np.concatenate(self.dtrajs), np.concatenate(self.bias_matrices))
+        The data is restructured to allow parallelization over Markov states. The dtrajs are only used to see
+        which Markov state the sample biases belong to. Ordering of the data doesn't matter (transition information is
+        already stored in the count matrices) so we can restructure the data.
+        After restructuring, each bias matrix in the bias_list corresponds to all sample biases for samples that fell
+        into the Markov state of which the index corresponds to the index of the bias matrix in the list.
+        Or: bias_list[i] contains biases for all samples that fell in Markov state i. """
+        bias_list = []
+        for markov_state in range(self.n_markov_states):
+            biases = []
+            for dtraj, bias_matrix in zip(self.dtrajs, self.bias_matrices):
+                indices = np.where(dtraj == markov_state)[0]
+                biases.append(bias_matrix[indices])
+
+            bias_list.append(np.concatenate(biases))
+        return tram.TRAMInput(self.state_counts, self.transition_counts, bias_list)
 
     @property
     def n_therm_states(self):
@@ -373,8 +395,12 @@ class TRAMDataset:
             if b.shape[1] != self.n_therm_states:
                 raise ValueError(
                     f"Second dimension of bias matrix {i} should be of size n_therm_states (={self.n_therm_states})")
-
+            
+            if b.shape[0] <= self.lagtime:
+                raise ValueError(f"""Trajectory {i} is shorter than the chosen lagtime. 
+                        There are no usable samples for this trajectory""")
             # If we were given ttrajs, do the same checks for those.
+        
         if self.ttrajs is not None:
             if len(self.ttrajs) != len(self.dtrajs):
                 raise ValueError("number of ttrajs is not equal to number of dtrajs.")
@@ -545,7 +571,7 @@ class TRAMDataset:
                               f"fragments that start at each replica exchange swap. Replica exchanges possibly occur "
                               f"within the span of the lag time.")
                 # there are no samples from this state that belong to the connected set. Make an empty count model.
-                count_models.append(TransitionCountModel(np.zeros(self.n_markov_states, self.n_markov_states)))
+                count_models.append(TransitionCountModel(np.zeros([self.n_markov_states, self.n_markov_states])))
             else:
                 # make a counts model for the samples that belong to the connected set.
                 traj_counts_model = estimator.fit_fetch(dtraj_fragments[k])
