@@ -3,6 +3,7 @@ from warnings import warn
 
 import numpy as np
 from scipy.integrate import odeint
+
 from sklearn.exceptions import ConvergenceWarning
 from sklearn.linear_model import LinearRegression, ridge_regression
 from sklearn.metrics import r2_score
@@ -34,7 +35,7 @@ class SINDyModel(Model):
         The feature library, :math:`\Theta`.
         It is assumed that this object has already been fit to the input data.
         The object should implement  :meth:`transform`
-        and :meth:`get_feature_names` methods.
+        and :meth:`get_feature_names_out` methods.
 
     coefficients : np.ndarray, shape (n_input_features, n_output_features)
         Coefficients giving the linear combination of basis functions to
@@ -130,7 +131,7 @@ class SINDyModel(Model):
 
         return scoring(y, self.predict(x), **scoring_kws)
 
-    def simulate(self, x0, t, integrator=odeint, integrator_kws={}):
+    def simulate(self, x0, t, integrator=odeint, integrator_kws=None):
         """
         Simulate the SINDy model forward in time.
 
@@ -148,7 +149,7 @@ class SINDyModel(Model):
             Function to use to integrate the system.
             Default is :code:`scipy.integrate.odeint`.
 
-        integrator_kws: dict, optional (default {})
+        integrator_kws: dict, optional (default None)
             Optional keyword arguments to pass to the integrator
 
         Returns
@@ -157,7 +158,10 @@ class SINDyModel(Model):
             Simulation results.
         """
 
-        def rhs(x, t):
+        if integrator_kws is None:
+            integrator_kws = {}
+
+        def rhs(x, _):
             return self.predict(x[np.newaxis, :])[0]
 
         return integrator(rhs, x0, t, **integrator_kws)
@@ -195,7 +199,7 @@ class SINDyModel(Model):
         equations : list of strings, length (n_input_features)
             List of model equations, with one for each input variable.
         """
-        feature_names = self.library.get_feature_names(
+        feature_names = self.library.get_feature_names_out(
             input_features=self.input_features
         )
         equation_list = [None] * len(self.coef_)
@@ -257,7 +261,7 @@ class SINDy(Estimator):
     library : library object, optional, default=None
         The candidate feature library, :math:`\Theta`.
         The object should implement a :meth:`fit`, :meth:`transform`,
-        and :meth:`get_feature_names` methods. It should also have
+        and :meth:`get_feature_names_out` methods. It should also have
         :attr:`n_input_features_` and :attr:`n_output_features_` attributes.
         By default a polynomial library of degree 2 is used.
 
@@ -325,7 +329,6 @@ class SINDy(Estimator):
         # Some differentiation methods produce nans near boundaries
         x, x_dot = drop_nan_rows(x, x_dot)
 
-        steps = [("features", self.library), ("model", self.optimizer)]
         if hasattr(self.library, 'fit'):
             self.library.fit(x)
 
@@ -434,7 +437,8 @@ class STLSQ(LinearRegression):
         # Do some preprocessing before fitting
         x_, y = check_X_y(x_, y, accept_sparse=[], y_numeric=True, multi_output=True)
 
-        x, y, X_offset, y_offset, X_scale = self._preprocess_data(
+        from sklearn.linear_model._base import _preprocess_data
+        x, y, X_offset, y_offset, X_scale = _preprocess_data(
             x_,
             y,
             fit_intercept=self.fit_intercept,
@@ -508,7 +512,8 @@ class STLSQ(LinearRegression):
         self.coef_ = coef
         self.ind_ = ind
 
-    def _sparse_coefficients(self, dim, ind, coef, threshold):
+    @staticmethod
+    def _sparse_coefficients(dim, ind, coef, threshold):
         r"""Perform thresholding of the weight vector(s)"""
         c = np.zeros(dim)
         c[ind] = coef
