@@ -20,9 +20,43 @@ template<typename dtype>
 constexpr static dtype logPrior() { return -std::numeric_limits<dtype>::infinity(); }
 }
 
+// Compute the sample likelihood from the traminput. This method only gets called by the TRAM struct, not by pybind.
+template<typename dtype>
+static const dtype computeSampleLikelihood(const TRAMInput<dtype> &input,
+                                           const np_array_nfc<dtype> &modifiedStateCountsLog) {
+    auto nThermStates = input.nThermStates();
+    const auto &cumNSamples = input.cumNSamples();
+    std::vector<dtype> sampleWeights(input.nSamples());
+
+    std::vector<ArrayBuffer<BiasMatrix<dtype>, 2>> biasMatrixBuf{begin(input.biasMatrices()),
+                                                                 end(input.biasMatrices())};
+    auto biasMatrixPtr = biasMatrixBuf.data();
+
+    ArrayBuffer<np_array_nfc<dtype>, 2> modifiedStateCountsLogBuf{modifiedStateCountsLog};
+    auto modifiedStateCountsLogPtr = &modifiedStateCountsLogBuf;
+
+    auto inputPtr = &input;
+    #pragma omp parallel for default(none) firstprivate(nThermStates, inputPtr, biasMatrixPtr, \
+                                                        modifiedStateCountsLogPtr, cumNSamples) shared(sampleWeights)
+    for (auto i = 0; i < inputPtr->nMarkovStates(); ++i) {
+        std::vector<dtype> scratch(nThermStates);
+        for (auto x = 0; x < inputPtr->nSamples(i); ++x) {
+            int o = 0;
+            for (StateIndex l = 0; l < nThermStates; ++l) {
+                if ((*modifiedStateCountsLogPtr)(l, i) > -std::numeric_limits<dtype>::infinity()) {
+                    scratch[o++] = (*modifiedStateCountsLogPtr)(l, i) - biasMatrixPtr[i](x, l);
+                }
+            }
+            auto logDivisor = numeric::kahan::logsumexp_sort_kahan_inplace(scratch.begin(), o);
+            sampleWeights[cumNSamples[i] + x] = -logDivisor;
+        }
+    }
+    return numeric::kahan::logsumexp_sort_kahan_inplace(sampleWeights.begin(), sampleWeights.end());
+}
 
 // natural logarithm of the statistical weight per sample, log \mu^k(x).
 // If thermState =-1, this is the unbiased statistical sample weight, log \mu(x).
+// This method gets called by pybind! Either directly or via the call to computeloglikelihood.
 template<typename dtype>
 static const std::vector<dtype> computeSampleWeightsLog(const DTraj &dtraj,
                                                         const BiasMatrix<dtype> &biasMatrix,
@@ -69,33 +103,48 @@ static const std::vector<dtype> computeSampleWeightsLog(const DTraj &dtraj,
 // i.e. the likelihood of observing the observed transitions plus for each thermodynamic state,
 // the free energy of that state times the state counts:
 // \sum_{i,j,k}c_{ij}^{(k)}\log p_{ij}^{(k)} + \sum_{i,k}N_{i}^{(k)}f_{i}^{(k)}
-template<typename EnergyBuffer, typename StateCountsBuffer, typename TransitionCountsBuffer, typename TransitionMatrixBuffer>
+template<typename dtype>
 static const auto
-computeDiscreteLikelihood(StateIndex nThermStates, StateIndex nMarkovStates, EnergyBuffer *biasedConfEnergies,
-                          StateCountsBuffer *stateCounts, TransitionCountsBuffer *transitionCounts,
-                          TransitionMatrixBuffer *transitionMatrices) {
-    using dtype = typename EnergyBuffer::value_type;
+computeDiscreteLikelihood(const np_array_nfc<dtype> &biasedConfEnergies,
+                          const CountsMatrix &stateCounts, const CountsMatrix &transitionCounts,
+                          const np_array_nfc<dtype> &transitionMatrices) {
+
+    auto nThermStates = static_cast<StateIndex>(biasedConfEnergies.shape(0));
+    auto nMarkovStates = static_cast<StateIndex>(biasedConfEnergies.shape(1));
+
+    // use threadsafe arraybuffers
+    ArrayBuffer<np_array_nfc<dtype>, 2> biasedConfEnergiesBuf{biasedConfEnergies};
+    auto biasedConfEnergiesPtr = &biasedConfEnergiesBuf;
+
+    ArrayBuffer<CountsMatrix, 3> transitionCountsBuf{transitionCounts};
+    auto transitionCountsPtr = &transitionCountsBuf;
+
+    ArrayBuffer<CountsMatrix, 2> stateCountsBuf{stateCounts};
+    auto stateCountsPtr = &stateCountsBuf;
+
+    ArrayBuffer<np_array_nfc<dtype>, 3> transitionMatricesBuf{transitionMatrices};
+    auto transitionMatricesPtr = &transitionMatricesBuf;
 
     dtype LL = 0;
 
-    #pragma omp parallel for default(none) firstprivate(nThermStates, nMarkovStates, transitionCounts, \
-                                                            transitionMatrices, stateCounts, \
-                                                            biasedConfEnergies) reduction(+:LL) collapse(2)
+    #pragma omp parallel for default(none) firstprivate(nThermStates, nMarkovStates, transitionCountsPtr, \
+                                                            transitionMatricesPtr, stateCountsPtr, \
+                                                            biasedConfEnergiesPtr) reduction(+:LL) collapse(2)
     for (StateIndex k = 0; k < nThermStates; ++k) {
         for (StateIndex i = 0; i < nMarkovStates; ++i) {
             // discrete sample log-likelihood \sum_{k=1}^K \sum_{i=1}^m N_i^k * f_i^k
-            if ((*stateCounts)(k, i) > 0) {
-                LL += ((*stateCounts)(k, i) + tram::detail::prior<dtype>()) * (*biasedConfEnergies)(k, i);
+            if ((*stateCountsPtr)(k, i) > 0) {
+                LL += ((*stateCountsPtr)(k, i) + tram::detail::prior<dtype>()) * (*biasedConfEnergiesPtr)(k, i);
             }
             // transition log-likelihood \sum_{k=1}^K \sum_{i,j=1}^m c_ij^k * log(p_ij^k)
             for (StateIndex j = 0; j < nMarkovStates; ++j) {
-                auto CKij = (*transitionCounts)(k, i, j);
+                auto CKij = (*transitionCountsPtr)(k, i, j);
                 if (CKij > 0) {
                     if (i == j) {
                         LL += (static_cast<dtype>(CKij) + tram::detail::prior<dtype>()) *
-                              std::log((*transitionMatrices)(k, i, j));
+                              std::log((*transitionMatricesPtr)(k, i, j));
                     } else {
-                        LL += CKij * std::log((*transitionMatrices)(k, i, j));
+                        LL += CKij * std::log((*transitionMatricesPtr)(k, i, j));
                     }
                 }
             }
@@ -108,6 +157,7 @@ computeDiscreteLikelihood(StateIndex nThermStates, StateIndex nMarkovStates, Ene
 // compute the log-likelihood of observing the input data under the current
 // biasedConfEnergies_. This computes the parameter-dependent part of the likelihood, i.e. the factor -b^k(x) is
 // omitted as it is constant.
+// This loglikelihood computation gets called py pybind! It expects only one (concatenated) trajectory.
 template<typename dtype>
 static const dtype computeLogLikelihood(const DTraj &dtraj,
                                         const BiasMatrix<dtype> &biasMatrix,
@@ -117,18 +167,10 @@ static const dtype computeLogLikelihood(const DTraj &dtraj,
                                         const CountsMatrix &stateCounts,
                                         const CountsMatrix &transitionCounts,
                                         const np_array_nfc<dtype> &transitionMatrices) {
-    auto nThermStates = static_cast<StateIndex>(biasedConfEnergies.shape(0));
-    auto nMarkovStates = static_cast<StateIndex>(biasedConfEnergies.shape(1));
-
-    // use threadsafe arraybuffers
-    ArrayBuffer<np_array_nfc<dtype>, 2> biasedConfEnergiesBuf{biasedConfEnergies};
-    ArrayBuffer<CountsMatrix, 3> transitionCountsBuf{transitionCounts};
-    ArrayBuffer<CountsMatrix, 2> stateCountsBuf{stateCounts};
-    ArrayBuffer<np_array_nfc<dtype>, 3> transitionMatricesBuf{transitionMatrices};
 
     // first get likelihood of all discrete quantities (transition likelihood and free energies times state counts)
-    auto logLikelihood = computeDiscreteLikelihood(nThermStates, nMarkovStates, &biasedConfEnergiesBuf,
-                                                   &stateCountsBuf, &transitionCountsBuf, &transitionMatricesBuf);
+    auto logLikelihood = computeDiscreteLikelihood(biasedConfEnergies, stateCounts, transitionCounts,
+                                                   transitionMatrices);
 
     // then compute log of all sample weights, and add to log likelihood.
     auto sampleWeights = computeSampleWeightsLog(dtraj, biasMatrix, thermStateEnergies, modifiedStateCountsLog, -1);
@@ -155,12 +197,12 @@ public:
               statVectors_(ExchangeableArray<dtype, 2>(std::vector({nThermStates_, nMarkovStates_}), 0.)),
               scratch_(std::unique_ptr<dtype[]>(new dtype[std::max(nMarkovStates_, nThermStates_)])) {
 
-            std::copy(lagrangianMultLog.data(), lagrangianMultLog.data() + lagrangianMultLog.size(),
+        std::copy(lagrangianMultLog.data(), lagrangianMultLog.data() + lagrangianMultLog.size(),
                   lagrangianMultLog_.first()->mutable_data());
-            std::copy(biasedConfEnergies.data(), biasedConfEnergies.data() + biasedConfEnergies.size(),
-                      biasedConfEnergies_.mutable_data());
-            std::copy(modifiedStateCountsLog.data(), modifiedStateCountsLog.data() + modifiedStateCountsLog.size(),
-                      modifiedStateCountsLog_.mutable_data());
+        std::copy(biasedConfEnergies.data(), biasedConfEnergies.data() + biasedConfEnergies.size(),
+                  biasedConfEnergies_.mutable_data());
+        std::copy(modifiedStateCountsLog.data(), modifiedStateCountsLog.data() + modifiedStateCountsLog.size(),
+                  modifiedStateCountsLog_.mutable_data());
     }
 
     const auto &biasedConfEnergies() const {
@@ -219,23 +261,25 @@ public:
             if (trackLogLikelihoods) {
                 // log likelihood depends on transition matrices. Compute them first.
                 computeTransitionMatrices();
-                logLikelihood = computeLogLikelihood(input_->dtraj(), input_->biasMatrix(),
-                                                     biasedConfEnergies_, modifiedStateCountsLog_,
-                                                     *thermStateEnergies_.first(), input_->stateCounts(),
-                                                     input_->transitionCounts(), transitionMatrices_);
+                logLikelihood = computeDiscreteLikelihood(biasedConfEnergies_, input_->stateCounts(),
+                                                          input_->transitionCounts(), transitionMatrices_) +
+                                computeSampleLikelihood(*input_, modifiedStateCountsLog_);
             }
 
             // Send convergence info back to user by calling a python callback function
-            if (callback != nullptr && callbackInterval > 0 && iterationCount % callbackInterval == 0) {
+            // Also if we have converged; in that case we want one last update of the final error to the user.
+	    if ((callback != nullptr && callbackInterval > 0 && iterationCount % callbackInterval == 0) 
+			    || iterationError < maxErr) {
                 py::gil_scoped_acquire guard;
                 (*callback)(callbackInterval, iterationError, logLikelihood);
-            }
-
-            if (iterationError < maxErr) {
-                // We have converged!
-                break;
-            }
+	
+                if (iterationError < maxErr) {
+                    // We have converged!
+ 		    break;
+                }
+	    }
         }
+
         // Done iterating. Compute all energies for the thermodynamic states and markov states.
         updateMarkovStateEnergies();
         updateThermStateEnergies();
@@ -328,29 +372,32 @@ private:
         std::fill(biasedConfEnergies_.mutable_data(), biasedConfEnergies_.mutable_data() +
                                                       biasedConfEnergies_.size(), inf);
 
-        auto biasMatrixBuf = input_->biasMatrixBuf();
-        auto dtrajBuf = input_->dtrajBuf();
+        std::vector<ArrayBuffer<BiasMatrix<dtype>, 2>> biasMatrixBuffers(input_->biasMatrices().begin(),
+                                                                         input_->biasMatrices().end());
+        auto biasMatrixPtr = biasMatrixBuffers.data();
 
         auto biasedConfEnergiesBuf = biasedConfEnergies_.template mutable_unchecked<2>();
         auto modifiedStateCountsLogBuf = modifiedStateCountsLog_.template unchecked<2>();
 
-        auto *scratch = scratch_.get();
+        auto nThermStates = nThermStates_, nMarkovStates = nMarkovStates_;
+        auto input = input_;
+        #pragma omp parallel for default(none) firstprivate(nMarkovStates, nThermStates, input, biasMatrixPtr, \
+                                                            biasedConfEnergiesBuf, modifiedStateCountsLogBuf)
+        for (StateIndex i = 0; i < nMarkovStates; ++i) {
+            std::vector<dtype> scratch(nThermStates);
 
-        // assume that biasedConfEnergies_ have been set to INF by the caller in the first call
-        for (std::int32_t x = 0; x < input_->nSamples(); ++x) {
-            auto i = dtrajBuf(x);
-            if (i >= 0) { // skip frames that have negative Markov state indices
+            for (std::int32_t x = 0; x < input->nSamples(i); ++x) {
                 std::size_t o = 0;
-                for (StateIndex k = 0; k < nThermStates_; ++k) {
+                for (StateIndex k = 0; k < nThermStates; ++k) {
                     if (modifiedStateCountsLogBuf(k, i) > -inf) {
-                        scratch[o++] = modifiedStateCountsLogBuf(k, i) - biasMatrixBuf(x, k);
+                        scratch[o++] = modifiedStateCountsLogBuf(k, i) - biasMatrixPtr[i](x, k);
                     }
                 }
-                dtype divisor = numeric::kahan::logsumexp_sort_kahan_inplace(scratch, o);
+                dtype divisor = numeric::kahan::logsumexp_sort_kahan_inplace(scratch.begin(), o);
 
-                for (StateIndex k = 0; k < nThermStates_; ++k) {
+                for (StateIndex k = 0; k < nThermStates; ++k) {
                     biasedConfEnergiesBuf(k, i) = -numeric::kahan::logsumexp_pair(
-                            -biasedConfEnergiesBuf(k, i), -(divisor + biasMatrixBuf(x, k)));
+                            -biasedConfEnergiesBuf(k, i), -(divisor + biasMatrixPtr[i](x, k)));
                 }
             }
         }
@@ -440,23 +487,29 @@ private:
         // first reset all confirmation energies to infinity
         std::fill(markovStateEnergies_.mutable_data(), markovStateEnergies_.mutable_data() + nMarkovStates_, inf);
 
-        auto modifiedStateCountsLogBuf = modifiedStateCountsLog_.template unchecked<2>();
+        std::vector<ArrayBuffer<BiasMatrix<dtype>, 2>> biasMatrixBuffers(input_->biasMatrices().begin(),
+                                                                         input_->biasMatrices().end());
+        auto biasMatrixPtr = biasMatrixBuffers.data();
+
         auto markovStateEnergiesBuf = markovStateEnergies_.template mutable_unchecked<1>();
+        auto modifiedStateCountsLogBuf = modifiedStateCountsLog_.template unchecked<2>();
 
-        auto dtrajBuf = input_->dtrajBuf();
-        auto biasMatrixBuf = input_->biasMatrixBuf();
-
+        auto nThermStates = nThermStates_, nMarkovStates = nMarkovStates_;
+        auto input = input_;
         // assume that markovStateEnergies_ were set to INF by the caller on the first call
-        for (auto x = 0; x < input_->nSamples(); ++x) {
-            std::int32_t i = dtrajBuf(x);
-            if (i >= 0) { // skip negative state indices
+        #pragma omp parallel for default(none) firstprivate(nMarkovStates, nThermStates, input, biasMatrixPtr, \
+                                                            markovStateEnergiesBuf, modifiedStateCountsLogBuf)
+        for (StateIndex i = 0; i < nMarkovStates; ++i) {
+            std::vector<dtype> scratch(nThermStates);
+
+            for (auto x = 0; x < input->nSamples(i); ++x) {
                 std::size_t o = 0;
-                for (StateIndex k = 0; k < nThermStates_; ++k) {
+                for (StateIndex k = 0; k < nThermStates; ++k) {
                     if (modifiedStateCountsLogBuf(k, i) > -inf) {
-                        scratch_[o++] = modifiedStateCountsLogBuf(k, i) - biasMatrixBuf(x, k);
+                        scratch[o++] = modifiedStateCountsLogBuf(k, i) - biasMatrixPtr[i](x, k);
                     }
                 }
-                dtype divisor = numeric::kahan::logsumexp_sort_kahan_inplace(scratch_.get(), o);
+                dtype divisor = numeric::kahan::logsumexp_sort_kahan_inplace(scratch.begin(), o);
                 markovStateEnergiesBuf(i) = -numeric::kahan::logsumexp_pair(-markovStateEnergiesBuf(i), -divisor);
             }
         }
@@ -489,7 +542,7 @@ private:
         auto ptr = biasedConfEnergies_.data();
         auto shift = *std::min_element(ptr, ptr + biasedConfEnergies_.size());
 
-        #pragma omp parallel for default(none) firstprivate(biasedConfEnergiesBuf, thermStateEnergiesBuf, shift)
+        #pragma omp parallel for default(none) firstprivate(nThermStates_, biasedConfEnergiesBuf, thermStateEnergiesBuf, shift)
         for (StateIndex k = 0; k < nThermStates_; ++k) {
             thermStateEnergiesBuf(k) -= shift;
 
@@ -594,7 +647,7 @@ private:
 };
 
 
-template <typename dtype>
+template<typename dtype>
 np_array_nfc<dtype> initLagrangianMult(const CountsMatrix &transitionCounts) {
     auto nThermStates = static_cast<StateIndex>(transitionCounts.shape(0));
     auto nMarkovStates = static_cast<StateIndex>(transitionCounts.shape(1));
