@@ -1,6 +1,29 @@
 r"""
 .. currentmodule: deeptime.util
 
+===============================================================================
+Model validation utils
+===============================================================================
+
+We currently offer an implied timescales check and a test of the Chapman-Kolmogorov equation:
+
+.. autosummary::
+    :toctree: generated/
+    :template: class_nomodule.rst
+
+    validation.implied_timescales
+    validation.ck_test
+
+
+with corresponding result objects
+
+.. autosummary::
+    :toctree: generated/
+    :template: class_nomodule.rst
+
+    validation.ImpliedTimescales
+    validation.ChapmanKolmogorovTest
+
 ===============================================================================
 Data utilities
 ===============================================================================
@@ -24,9 +47,10 @@ Statistics utilities
     :toctree: generated/
     :template: class_nomodule.rst
 
+    energy2d
+    EnergyLandscape2d
     QuantityStatistics
     confidence_interval
-    LaggedModelValidator
 
 ===============================================================================
 Type utilities
@@ -51,6 +75,7 @@ Other utilities
     parallel.handle_n_jobs
     decorators.cached_property
     decorators.plotting_function
+    decorators.deprecated_argument
 
     callbacks.supports_progress_interface
     callbacks.ProgressCallback
@@ -59,10 +84,10 @@ Other utilities
     platform.handle_progress_bar
 """
 
-from .stats import QuantityStatistics, confidence_interval
-from ._validation import LaggedModelValidator
+from .stats import QuantityStatistics, confidence_interval, energy2d, EnergyLandscape2d
 
 from . import data
 from . import types
 from . import callbacks
 from . import platform
+from . import validation
